@@ -3,6 +3,7 @@ import logging
 import gobject
 import gtk
 import deskbar
+import deskbar.interfaces.Match
 from deskbar.core.GconfStore import GconfStore
 from deskbar.core.ModuleInstaller import ModuleInstaller
 from deskbar.core.ModuleLoader import ModuleLoader
@@ -168,7 +169,7 @@ class CoreImpl(deskbar.interfaces.Core):
         if not self._keybinder.bind(binding):
             LOGGER.error("Keybinding is already in use")
         else:
-            LOGGER.info("Successfully binded Deskbar to %s" % binding)
+            LOGGER.info("Successfully binded Deskbar to %s", binding)
     
     def set_min_chars(self, number):
         self._gconf.set_min_chars(number)
@@ -354,6 +355,9 @@ class CoreImpl(deskbar.interfaces.Core):
             
     def forward_query_ready(self, handler, query, matches):
         if query == self._last_query and matches != None and len(matches) > 0 and not self._stop_queries:
+            for match in matches:
+                if not isinstance(match, deskbar.interfaces.Match):
+                    raise TypeError("Handler %r returned an invalid match: %r", handler,  match)
             self._emit_query_ready(matches)
     
     def update_gconf(self):
@@ -364,11 +368,10 @@ class CoreImpl(deskbar.interfaces.Core):
     def _on_default_browser_changed(self, gconfstore, new_browser):
         new_browser = new_browser.split(" ")[0]
         
-        if new_browser.find("firefox") != -1 or new_browser.find("iceweasel") != -1 \
-            or new_browser.find("iceweasel") != -1:
+        if new_browser == "firefox":
             new_browser = "mozilla"
         
-        if new_browser.find("epiphany") != -1:
+        if new_browser == "epiphany":
             old_browser = "mozilla"
         elif new_browser == "mozilla":
             old_browser = "epiphany"
@@ -411,10 +414,7 @@ class CoreImpl(deskbar.interfaces.Core):
         for module in enabled_browser_modules:
             new_module_name = module.replace(old_browser.capitalize(), new_browser.capitalize())
             new_module = self._module_list.get_module_instance_from_name(new_module_name)
-            
-            # If new_module is None the module has missing requirements
-            if new_module != None:
-                # If async is True then self.update_gconf() may be run before the modules were initialized
-                self.initialize_module(new_module, async=False)
+            # If async is True then self.update_gconf() may be run before the modules were initialized
+            self.initialize_module(new_module, async=False)
         
         self.update_gconf()
