@@ -8,20 +8,27 @@ class View(object):
         gtk.window_set_default_icon_name("deskbar-applet")
         
     def clear_results(self):
-        raise NotImplemented
+        raise NotImplementedError
+        
+    def clear_actions(self):
+        raise NotImplementedError
     
     def clear_query(self):
-        raise NotImplemented
+        raise NotImplementedError
     
     def clear_all(self):
         self.clear_query()
         self.clear_results()
+        self.clear_actions()
+    
+    def show_results(self):
+        raise NotImplementedError
     
     def get_toplevel(self):
-        raise NotImplemented
+        raise NotImplementedError
     
     def get_entry(self):
-        raise NotImplemented
+        raise NotImplementedError
     
     def show_history(self, value):
         raise NotImplementedError
@@ -33,4 +40,7 @@ class View(object):
         raise NotImplementedError
        
     def display_actions(self, actions, qstring):
+        raise NotImplementedError
+    
+    def mark_history_empty(self, val):
         raise NotImplementedError
\ No newline at end of file
