@@ -5,14 +5,17 @@ import traceback
 from gettext import gettext as _
 from os.path import join
 import deskbar
-from deskbar.core.updater.NewStuffUpdater import NewStuffUpdater
+from deskbar.core.updater.Capuchin import *
 from deskbar.core.ModuleList import WebModuleList
 from deskbar.ui.preferences.AccelEntry import AccelEntry
 from deskbar.ui.preferences.ErrorDialog import ErrorDialog
 from deskbar.ui.preferences.ModuleListView import ModuleListView, DisabledModuleListView, WebModuleListView
+from deskbar.ui.preferences.ProgressbarDialog import ProgressbarDialog
 import dbus
-if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
-    import dbus.glib
+import time
+import dbus.glib
+    
+DESKBAR_CAPUCHIN_REPO = "http://www.gnome.org/~sebp/deskbar/deskbar-applet.xml"
 
 class InfoBox(gtk.HBox):
     
@@ -32,6 +35,7 @@ class DeskbarPreferences:
     
     def __init__(self, model):
         self._model = model
+        self._model.connect("initialized", self.on_modules_initialized)
         
         self.module_list = self._model.get_module_list()
     
@@ -39,8 +43,12 @@ class DeskbarPreferences:
         
         self.dialog = self.glade.get_widget("preferences")
         
-        # Since newstuff is optional we have to check if self.newstuff is None each time we use it
-        self.newstuff = None
+        # Since capuchin is optional we have to check if self.capuchin is None each time we use it
+        self.__capuchin = None
+        self.has_capuchin = False
+        self.__capuchin_updated = False
+        self.progessdialog = None
+        self.__capuchin_installing = False
         
         self.keybinding = self._model.get_keybinding()
         
@@ -53,8 +61,8 @@ class DeskbarPreferences:
         
         self.__setup_disabled_modules_tab()
         
-        # Setup new-stuff-manager
-        self.__enable_newstuffmanager( self.__is_nsm_available() )
+        # Setup capuchin
+        self.__setup_updater()
           
         self.__select_first_tab()
         
@@ -76,7 +84,8 @@ class DeskbarPreferences:
         self.more_button_callback = None
         self.reload_button = self.glade.get_widget("reload")
         self.reload_button.connect("clicked", self.on_reload_button_clicked)
-        self.reload_button.set_tooltip_text(_("Reload all extensions"))
+        self.reload_button_tooltip = gtk.Tooltips ()
+        self.reload_button_tooltip.set_tip(self.reload_button, _("Reload all extensions"))
 
         # Info are at the bottom
         self.info_area = self.glade.get_widget("info_area")
@@ -99,13 +108,14 @@ class DeskbarPreferences:
         self.keyboard_shortcut_entry = AccelEntry()
         self.keyboard_shortcut_entry.connect('accel-edited', self.on_keyboard_shortcut_entry_changed)
         self.glade.get_widget("keybinding_entry_container").pack_start(self.keyboard_shortcut_entry.get_widget(), False)
-        #self.keybinding_notify_id = deskbar.GCONF_CLIENT.notify_add(applet.prefs.GCONF_KEYBINDING, lambda x, y, z, a: self.on_config_keybinding(z.value))
         
         self.use_selection = self._model.get_use_selection()
         self.use_selection_box = self.glade.get_widget("use_selection")
         self.use_selection_box.connect('toggled', self.on_use_selection_toggled)
-        #self.use_selection_id = deskbar.GCONF_CLIENT.notify_add(applet.prefs.GCONF_USE_SELECTION, lambda x, y, z, a: self.on_config_use_selection(z.value))
 
+        self.sticktopanel_checkbox = self.glade.get_widget("sticktopanel_checkbox")
+        self.sticktopanel_checkbox.connect("toggled", self.on_ui_changed)
+       
     def __setup_drag_and_drop(self):
         big_box = self.glade.get_widget("big_box")
         self.TARGET_URI_LIST, self.TARGET_NS_URL = range(2)
@@ -129,26 +139,38 @@ class DeskbarPreferences:
         self.disabledhandlers_box = self.glade.get_widget("disabledhandlers_box")
 
     def __select_first_tab(self):
-         # Select first tab
+        """ Select first tab """
         notebook = self.glade.get_widget("notebook1")
         current = notebook.get_current_page()
         if (current != 0):
             for i in range(current):
                 notebook.prev_page()
 
-    def __is_nsm_available(self):
-        bus = dbus.SessionBus()
-        proxy = bus.get_object('org.freedesktop.DBus', '/org/freedesktop/DBus')
-        _dbus = dbus.Interface(proxy, 'org.freedesktop.DBus')
-        _dbus.ReloadConfig()
-        bus_names = _dbus.ListActivatableNames()
-        return (NewStuffUpdater.NEW_STUFF_SERVICE in bus_names)
-
-    def __enable_newstuffmanager(self, status):
-        if status:
+    def __setup_updater(self):
+        if is_capuchin_available():
+            self.has_capuchin = True
+            
             self.web_module_list = WebModuleList()
             
+            self.ROW_SEPERATOR_STRING = "<-->"
+            self.ALL_EXTENSIONS_TEXT = _("All Extensions")
+            
+            self.combobox_tags = self.glade.get_widget("combobox_tags")
+            self.combobox_tags.set_row_separator_func( lambda model, iter: model[iter][0] == self.ROW_SEPERATOR_STRING )  
+            
+            tag_cell = gtk.CellRendererText ()
+            self.combobox_tags.pack_start (tag_cell)
+            self.combobox_tags.add_attribute (tag_cell, 'text', 0)
+            
+            self.tags_list = gtk.ListStore (str)
+            self.tags_list.set_sort_column_id (0, gtk.SORT_ASCENDING)
+            self.tags_list.set_sort_func (0, self._tags_sort_func)
+            
+            self.combobox_tags.set_model (self.tags_list)
+            self.combobox_tags.connect ("changed", self.on_combobox_tags_changed)
+            
             container = self.glade.get_widget("newhandlers")
+            
             self.webmoduleview = WebModuleListView(self.web_module_list)
             self.webmoduleview.get_selection().connect("changed", self.on_webmodule_selected)
             self.web_module_list.connect('row-changed', lambda list, path, iter: self.on_webmodule_selected    (self.webmoduleview.get_selection()))
@@ -173,16 +195,55 @@ class DeskbarPreferences:
             self.glade.get_widget("check").destroy()
             self.glade.get_widget("update").destroy()
     
-    def _call_on_newstuffmanager(self, method, *args):
-        if self.newstuff == None:
-            self.newstuff = NewStuffUpdater()
-            self.newstuff.connect('ready', lambda s: getattr(self.newstuff, method)(*args) )
-            self.newstuff.connect('error', self.on_newstuff_error)
-            self.newstuff.connect('new-modules-available', self.on_new_modules_available)
-            self.newstuff.connect('updates-available', self.on_updates_available)
+    def _get_capuchin_instance(self):
+        if self.__capuchin == None:
+            manager = AppObjectManager ()
+            self.__capuchin = manager.get_app_object (DESKBAR_CAPUCHIN_REPO)
+            self.__capuchin.connect ('install-finished', self._on_install_finished)
+            self.__capuchin.connect ('status', self._on_status)
+            self.__capuchin.update (False)
+            
+            if self.progessdialog != None:
+                self.progessdialog.destroy()
+                self.progessdialog = None
+
+        return self.__capuchin
+       
+    def _show_error_dialog(self, error):
+          """
+          An error message will be displayed in a MessageDialog
+          """
+          dialog = ErrorDialog(self.dialog, _("A problem occured"), error)
+          dialog.run()
+    
+    def _show_module_installed_dialog(self):
+        dialog = gtk.MessageDialog(parent=self.dialog,
+                       flags=gtk.DIALOG_DESTROY_WITH_PARENT,
+                       type=gtk.MESSAGE_INFO,
+                       buttons=gtk.BUTTONS_OK,
+                       message_format=_("Extension has been installed successfully"))
+        dialog.set_modal(True)
+        dialog.connect ('response', lambda w, r: w.destroy())
+        dialog.show_all ()
+        
+    def _tags_sort_func(self, model, iter1, iter2):
+        val1 = model[iter1][0]
+        val2 = model[iter2][0]
+        if val1 == self.ALL_EXTENSIONS_TEXT:
+            # Always at top
+            return -1
+        elif val2 == self.ALL_EXTENSIONS_TEXT:
+            # Always at top
+            return 1
         else:
-            getattr(self.newstuff, method)(*args)
-   
+            # Sort alphabetically
+            if val1 < val2:
+                return -1
+            elif val1 > val2:
+                return 1
+            else:
+                return 0
+          
     def show_run_hide(self, parent):
         self.dialog.set_screen(parent.get_screen())
         self.dialog.show_all()
@@ -190,10 +251,11 @@ class DeskbarPreferences:
         self.dialog.connect("response", self.on_dialog_response)
     
     def on_dialog_response(self, dialog, response):
-        self.update_gconf()    
+        self._model.update_gconf()
         self.dialog.destroy()
-        if self.newstuff != None:
-            self.newstuff.close()
+        # Check if capuchin service is available and if actually used it
+        if self.has_capuchin and self.__capuchin != None:
+            self.__capuchin.close()
         
     def sync_ui(self):
         if self.keybinding != None:
@@ -202,6 +264,8 @@ class DeskbarPreferences:
             self.keyboard_shortcut_entry.set_accelerator_name("<Alt>F3")
         
         self.use_selection_box.set_active(self.use_selection)
+         
+        self.sticktopanel_checkbox.set_active( self._model.get_ui_name() == deskbar.BUTTON_UI_NAME)
    
     def on_hide_after_action_toggled(self, toggle):
         self._model.set_hide_after_action(toggle.get_active())
@@ -219,26 +283,42 @@ class DeskbarPreferences:
 
     def on_use_selection_toggled(self, toggle):
         self._model.set_use_selection(toggle.get_active())
+
+    def on_use_newstyleui_toggled(self, toggle):
+        self._model.set_use_newstyleui(toggle.get_active())
         
     def on_more_button_clicked(self, button):
         if self.more_button_callback != None:
             self.more_button_callback(self.dialog)
 
     def on_reload_button_clicked(self, button):
+        self.reload_button.set_sensitive(False)
         self._model.reload_all_modules()
     
     def on_module_selected(self, selection):
-        module_context = self.moduleview.get_selected_module()
-        
-        if module_context != None:
-            self.check_requirements(module_context)
-            
+        """
+        Check if update for module is available
+        and set button (in)sensitive
+        """
+        module = self.moduleview.get_selected_module()
+         
+        if module != None:
+            self.check_requirements(module)
+           
+        # Check if module is not None, because the signal
+        # is emitted when the user moves a module, too  
         # Check if we can update
-        if self.newstuff != None:
-            #TODO: Save information whether update is available
-            self.update.set_sensitive(module_context != None)                
+        if self.has_capuchin:
+            if module != None:
+                self.update.set_sensitive(module.is_updateable())
+            else:
+                self.update.set_sensitive(False)
     
     def set_buttons(self, selection):
+        """
+        Set top/up/down/bottom sensitive according
+        to the selected module
+        """
         model, iter = selection.get_selected()
         if iter == None:
             self.__set_top_buttons_sensitive(False)
@@ -310,63 +390,169 @@ class DeskbarPreferences:
             self._model.stop_module (module, False)
         else:
             self._model.initialize_module (module, False)
-        self.update_gconf()
-       
-    def on_newstuff_error(self, newstuff, error):
-          """
-          Called if a connection to the repository failed
-          
-          An error message will be displayed in a MessageDialog
-          and C{self.newstuff} is reset
-          
-          @type error: dbus_bindings.DBusException instance
-          """
-          dialog = ErrorDialog(self.dialog, _("A problem occured"), error)
-          dialog.run()
-          self.newstuff.close()
-          self.newstuff = None
-          
+        self._model.update_gconf()
+    
     def on_check_handlers(self, button):
-        #Update all handlers
-        current_modules = []
+        """
+        Check if updates for the installed modules are available
+        """
+        installed_modules = set()
         for mod in self.module_list:
-            current_modules.append((mod.get_id(), mod.INFOS["version"]))
-        self._call_on_newstuffmanager("fetch_updates", current_modules)
+            installed_modules.add( mod.get_id() )
+        
+        try:
+            repo_plugins = set (self._get_capuchin_instance().get_available_plugins())
+            
+            # Get modules that are installed locally and are in the repository
+            updateable_modules = repo_plugins & installed_modules 
+                
+            current_modules = []
+            for mod in self.module_list:
+                if mod.get_id() in updateable_modules:
+                    current_modules.append( PluginInfo(mod.get_id(), mod.INFOS["version"]) )
+            
+            updates = self._get_capuchin_instance().get_available_updates (current_modules)
+           
+            for modid in updates:
+                iter = self.module_list.get_iter_from_module_id (modid)
+                self.module_list[iter][self.module_list.MODULE_CTX_COL].set_updateable (True)
+                self.module_list.set_module_update_available(iter, True)
+        except dbus.exceptions.DBusException, e:
+            self._show_error_dialog(e)
+            self.__capuchin.close()
+            self.__capuchin = None
             
     def on_check_new_extensions(self, button):
-        current_modules = []
-        for mod in self.module_list:
-            current_modules.append(mod.get_id())
-        self._call_on_newstuffmanager("fetch_new_modules", current_modules)
-        
-    def on_new_modules_available(self, newstuff, modules):
-        for mod in modules:
-            self.web_module_list.add(mod)
-        
-    def on_updates_available(self, newstuff, modules):
-        for modid, desc in modules:
-            self.module_list.set_module_update_available(modid)
+        self._get_tags()
+        self._get_new_modules()
         
     def on_update_handler(self, button):
+        """ Update the selected module """
         module = self.moduleview.get_selected_module()
         if module != None:
-            # Trigger module update
-            if self.newstuff != None:
-                self.newstuff.install_module(module.get_id())
+            self.__capuchin_installing = False
+            self._capuchin_install(module.get_id())
             button.set_sensitive(False)
         
     def on_install_handler(self, button):
-        # Install the selected new handler
+        """ Install the selected new handler """
         mod_id = self.webmoduleview.get_selected_module_id()
+        self.__capuchin_installing = True
+        self._capuchin_install(mod_id)
+        button.set_sensitive(False)
+        
+    def on_combobox_tags_changed (self, combobox):
+        iter = combobox.get_active_iter()
+        if iter == None:
+            return
+        
+        selected_tag = self.tags_list[iter][0]
+        
+        if selected_tag == self.ALL_EXTENSIONS_TEXT:
+            self.web_module_list.clear ()
+            self._get_new_modules()
+        else:
+            try:
+                plugins = self._get_capuchin_instance().get_plugins_with_tag (selected_tag)
+                self.web_module_list.clear ()
+                for mod_id in plugins:
+                    self._add_module_infos_to_web_modules (mod_id)
+            except dbus.exceptions.DBusException, e:
+                self._show_error_dialog(e)
+                self.__capuchin.close()
+                self.__capuchin = None
+            
+    def _capuchin_install(self, mod_id):
         if mod_id != None:
-            if self.newstuff != None:
-                self.newstuff.install_module(mod_id)
-            button.set_sensitive(False)
+            if self.has_capuchin:
+                try:
+                    self._get_capuchin_instance().install(mod_id)
+                except dbus.exceptions.DBusException, e:
+                    self._show_error_dialog(e)
+                    self.__capuchin.close()
+                    self.__capuchin = None
      
+    def _get_tags (self):
+        self.tags_list.clear()
+        self.tags_list.append ([self.ALL_EXTENSIONS_TEXT])
+        self.tags_list.append ([self.ROW_SEPERATOR_STRING])
+            
+        try:
+            for tag in self._get_capuchin_instance().get_tags ():
+                self.tags_list.append ( [tag] )
+        except dbus.exceptions.DBusException, e:
+            self._show_error_dialog(e)
+            self.__capuchin.close()
+            self.__capuchin = None
+            
+        self.combobox_tags.set_active (0) # Set to all
+    
+    def _get_new_modules (self):
+        """
+        Display a list of modules that aren't installed, yet
+        """
+        local_modules = set()
+        for mod in self.module_list:
+            local_modules.add(mod.get_id())
+        
+        try:
+            repo_modules = set (self._get_capuchin_instance().get_available_plugins())
+            
+            new_modules = repo_modules - local_modules
+            
+            for mod_id in new_modules:
+                self._add_module_infos_to_web_modules (mod_id)
+        except dbus.exceptions.DBusException, e:
+            self._show_error_dialog(e)
+            self.__capuchin.close()
+            self.__capuchin = None
+            
+    def _add_module_infos_to_web_modules (self, mod_id):
+        mod_name, mod_desc = self._get_capuchin_instance().get_plugin_infos (mod_id)
+        self.web_module_list.add(mod_id, mod_name, mod_desc)
+   
     def on_webmodule_selected(self, selection):
         mod_id = self.webmoduleview.get_selected_module_id()
         self.install.set_sensitive(mod_id != None)
-              
+    
+    def _on_status(self, appobject, action, plugin_id, progress, speed):
+        if action == ACTION_UPDATING_REPO:
+            action_text = _("Retrieving the extension index")
+        elif action == ACTION_DOWNLOADING_PLUGIN:
+            action_text = _("Downloading extension")
+        elif action == ACTION_EXTRACTING_PLUGIN:
+            action_text = _("Extracting archive")
+        else:
+            return
+        
+        if self.progessdialog == None:
+            # Create new dialog
+            self.progessdialog = ProgressbarDialog (self.dialog)
+            self.progessdialog.set_text ( _("Installing extension"),
+                                      _("The extension will be downloaded and installed.") )
+            self.progessdialog.run_nonblocked ()
+        
+        self.progessdialog.set_current_operation (action_text)
+        self.progessdialog.set_fraction (progress)
+    
+    def _on_install_finished(self, appobject, plugin_id):
+        self.progessdialog.destroy ()
+        self.progessdialog = None
+        
+        # Remove from webmodulelist
+        if self.__capuchin_installing:
+            # We where installing a new module
+            model, iter = self.webmoduleview.get_selection().get_selected()
+            self.web_module_list.remove (iter)
+            self.__capuchin_installing = False
+        else:
+            # We updated a already installed module
+            model, iter = self.moduleview.get_selection().get_selected()
+            self.module_list[iter][self.module_list.MODULE_CTX_COL].set_updateable (False)
+            self.module_list.set_module_update_available(iter, False)
+        
+        self._show_module_installed_dialog()
+           
     def on_drag_motion(self, widget, drag_context, x, y, timestamp):
         return False
     
@@ -382,16 +568,13 @@ class DeskbarPreferences:
             data = selection.data.strip()
         try:
             self._model.install_module(data)
-            dialog = gtk.MessageDialog(parent=self.dialog,
-                       flags=gtk.DIALOG_DESTROY_WITH_PARENT,
-                       type=gtk.MESSAGE_INFO,
-                       buttons=gtk.BUTTONS_OK,
-                       message_format=_("Handler has been installed successfully"))
-            dialog.connect('response', lambda w, id: dialog.destroy())
+            self._show_module_installed_dialog()
         except Exception, e:
-            dialog = ErrorDialog(self.dialog, _("Handler could not be installed due a problem with the provided file"), traceback.format_exc() )
+            dialog = ErrorDialog(self.dialog,
+                                 _("Extension could not be installed due a problem with the provided file"),
+                                 traceback.format_exc() )
+            dialog.run()
         
-        dialog.run()
         return
     
     def on_button_top_clicked(self, button):
@@ -422,7 +605,11 @@ class DeskbarPreferences:
         self.set_buttons(self.moduleview.get_selection())
         self.moduleview.grab_focus()
         
-    def update_gconf(self):
-         # Update the gconf enabled modules settings
-        enabled_modules = [mod.__class__.__name__ for mod in self.module_list if mod.is_enabled()]
-        self._model.set_enabled_modules(enabled_modules)
+    def on_ui_changed(self, check):
+        if self.sticktopanel_checkbox.get_active():
+            self._model.set_ui_name(deskbar.BUTTON_UI_NAME)
+        else:
+            self._model.set_ui_name(deskbar.WINDOW_UI_NAME)
+            
+    def on_modules_initialized(self, model):
+        self.reload_button.set_sensitive(True)
